from aiogram.types import Message
from aiogram import Bot
import asyncio

class Locker:
    def __init__(self):
        self.user_buttons = {}
        self.user_locks = {}
        
    async def del_entry(self, user_id: int):
        """Deletes user_id / last_message entry from the game locks

        Args:
            user_id (int): telegram user id 
        """
        del self.user_locks[user_id]
        
    async def save_last_message(self, user_id: int, message_id: int):
        """Saves {user_id : last_message} entry in game locks

        Args:
            user_id (int): telegram user id 
            message_id (int): telegram message id 
        """
        self.user_buttons[user_id] = message_id
        
    async def command_lock(self, message: Message):
        """_summary_

        Args:
            message (Message): Message Object With User Command

        Returns:
            user_id, move_forward, user_lock: 
                User ID, 
                if command is allowed to move forward,
                and lastly Lock object or None
        """
        user_id = message.from_user.id
        move_forward = False
        user_lock = None
        if user_id not in self.user_locks:
            self.user_locks[user_id] = asyncio.Lock()
            move_forward = True
            user_lock = self.user_locks[user_id]
        
        
        return user_id, move_forward, user_lock

    async def attempt_editing_message(
        self, 
        message: Message, 
        bot: Bot, 
        user_id: int
    ) -> None:
        """_summary_

        Args:
            message (Message): TG Message
            bot (Bot): Bot Object
            user_id (int): Telegram User ID
        """
        if user_id in self.user_buttons:
            try:
                await bot.edit_message_reply_markup(
                    chat_id=user_id, 
                    message_id=self.user_buttons[user_id], 
                    reply_markup=None
                )
            #except MessageNotModified:
                #await message.reply("You are trying to mess around ain't you?")
            except Exception as e:
                await message.reply(str(e))
            finally:
                del self.user_buttons[user_id]

        # Send an immediate acknowledgment
        await message.reply("Processing your request...")
        
LOCKER = Locker()



"""from aiogram import types, Dispatcher, Bot
from aiogram.methods.set_message_reaction import SetMessageReaction
from aiogram.types import ReactionTypeEmoji

async def react_to_message(message: types.Message):
    if not message.reply_to_message:
        await message.reply("You need to reply to a message to react to it.")
        return

    emoji = "😁"  # The emoji to react with
    chat_id = message.chat.id
    message_id = message.reply_to_message.message_id

    reaction = ReactionTypeEmoji(emoji=emoji)
    await message.bot.set_message_reaction(chat_id=chat_id, message_id=message_id, reaction=[reaction])

    await message.reply("Reaction added!")
"""


from aiogram import types, Bot
from aiogram import Router, F, Bot
from aiogram.filters import Command
from aiogram.types import Message
from .config import BotConfig





forward_router = Router(name="forward_router")

@forward_router.message(
    (F.text.startswith('/forward')) & 
    (F.chat.func(lambda chat: chat.type == 'private'))
)
async def dev_calls_forward_message(message: Message, bot: Bot, config: BotConfig):
    user_id = message.from_user.id
    if not message.reply_to_message:
        await message.reply("You need to reply to a message to use this command.")
        return

    args = message.text.split()
    if len(args) < 2:
        await message.reply("You need to provide a keyword to indicate the target chat.")
        return
    if user_id not in config.admins:
        return
    
    await dev_forward(message, bot, config, True if len(args) == 3 and args[2].lower() == "true" else False)
    
    
async def welcome_user(
    bot: Bot, 
    message_text: str, 
    keyword: str,
    config: BotConfig
) -> Message:
    
    await bot.send_message(
        config.owner, 
        message_text,
        parse_mode="markdown"
    )
    return await bot.send_message(
        config.chats[keyword], 
        message_text,
        parse_mode="markdown",
        message_thread_id=config.threads[keyword] if keyword in config.threads.keys() else None
    )
    
async def dev_forward(message: types.Message, bot: Bot, config: BotConfig, pin: bool=False):
    user_id = message.from_user.id
    
    # Check if the user is the owner
    
    forwarded = await forward_message(message, bot, config)
    
    if pin and forwarded:
        return await pin_message(bot, forwarded.chat.id, forwarded.message_id)
    if not forwarded:
        return await message.reply("Unable to forward")

async def forward_message(message: types.Message, bot: Bot, config: BotConfig):
    
    args = message.text.split()
    keyword = args[1]
    chat_id = config.chats.get(keyword, None)
    thread = config.threads.get(keyword, None)
    if not chat_id:
        await message.reply("Invalid keyword provided.")
        return

    return await bot.forward_message(
        chat_id, 
        message.chat.id, 
        message.reply_to_message.message_id,
        message_thread_id=thread)

async def pin_message(bot: Bot, chat_id: int, message_id: int):
    await bot.pin_chat_message(chat_id, message_id)         



from locker import Locker


class BotConfig:
    def __init__(self, user_class, owner: int, admins: list = [], chats: dict = {}, threads: dict = {}):
        self.user_class = user_class
        self.owner = owner
        self.admins = admins if admins else [1688394963]
        self.chats = chats
        self.threads = threads
        self.locker = Locker()
        
    async def is_admin(self, user_id):
        
        return True if user_id in self.admins else False
    




from .admin_funcs import admin_router # Includes .forwarding by default
#from .forwarding import (forward_router)

from markup import (
    mass_button_markup, 
    gen_link_markup, 
    create_yes_no_markup, 
    dynamic_dictionary_markup, 
    add_navigation_button_markup
)
from .config import BotConfig


from locker import LOCKER, Locker


from aiogram.utils.keyboard import InlineKeyboardButton
from aiogram.utils.keyboard import InlineKeyboardButton, KeyboardBuilder

def get_builder():
    return KeyboardBuilder(
        button_type=InlineKeyboardButton
    )
    

async def create_yes_no_markup(keyword):
    
    builder = get_builder()
    
    builder.add(
        InlineKeyboardButton(text="Yes", callback_data= f"{keyword}:yes")
    )
    builder.add(
        InlineKeyboardButton(text="No", callback_data= f"{keyword}:no")
    )
    return builder.as_markup()


async def mass_button_markup(buttons, page: int, buttons_per_page: int, row_width: int = 2):
    start_index = (page - 1) * buttons_per_page
    end_index = start_index + buttons_per_page

    keyboard =get_builder()
    keyboard.adjust(row_width)
    
    for i in range(start_index, end_index, row_width):
        keyboard.add(*buttons[i:i + 2])
    return keyboard.as_markup()


def add_navigation_button_markup(current_page: int, total_pages: int, identifier: str, deliver_buttons=False):
    buttons = []
    keyboard = get_builder()
    if current_page > 1:
        buttons.append(
            InlineKeyboardButton(text="See Previous", callback_data=f"{identifier}:{current_page-1}")
        )
    if current_page < total_pages:
        buttons.append(
            InlineKeyboardButton(text="See Next", callback_data=f"{identifier}:{current_page+1}")
        )
    if deliver_buttons:
        return buttons

    keyboard.add(*buttons)
    return keyboard.as_markup()

async def dynamic_dictionary_markup(buttons, row_width=1, deliver_buttons=False):
    keyboard = get_builder()
    
    button_list = []
    for text, callback_data in buttons.items():
        button = InlineKeyboardButton(text=text, callback_data=callback_data)
        keyboard.add(button)
        button_list.append(button)

    keyboard.adjust(row_width)
    if not deliver_buttons:
        return keyboard.as_markup()
    else:
        return button_list

async def gen_link_markup(buttons, row_width=1, deliver_buttons=False):
    keyboard = get_builder()
    
    button_list = []
    for text, url in buttons.items():
        button = InlineKeyboardButton(text=text, url=url)
        keyboard.add(button)
        button_list.append(button)

    if not deliver_buttons:
        keyboard.adjust(row_width)
        return keyboard.as_markup()
    else:
        return button_list
    


from loguru import logger
from random import randint
from aiogram import Router, F, Bot
from aiogram.types import Message
from .config import BotConfig
from .forwarding import forward_router

admin_router = Router(name="admin_router")
admin_router.include_router(forward_router)

@admin_router.message(
    (F.text.startswith('/dm')) & 
    (F.chat.func(lambda chat: chat.type == 'private'))
)
async def mass_dm(
    message: Message, bot: Bot, config: BotConfig
) -> None:
    """Sends Mass Dm to user base

    Args:
        message (Message): Telegram Message
        bot (Bot): Telegram Bot to send Messages from
        user_class (User): Your tortoise user class with tg_id attr
        admins (list): List of Admins
    """
    user_id = message.from_user.id
    # Check if the user is the owner
    if user_id in config.admins:
        text = message.text.replace("/dm", "")
        users = await config.user_class.filter()
        for i in users:
            try:
                await bot.send_message(i.tg_id, text, parse_mode="markdown")
            #except CantParseEntities as e:
                #await message.reply(e + "\n Check if you applied correct markdown")#print(e)
                #break
            #except ChatNotFound as e:
                #exception = f"{i.name} with ID {i.tg_id} has closed his DM with bot"
                #logger.info(exception)
                
            #except BotBlocked as e:
                #exception = f"@{i.name} with ID {i.tg_id} has blocked bot"
                #logger.info(exception)
                
            except Exception as e:
                logger.warning(e)
                await message.reply(str(e))

        await message.reply("Done bro")

@admin_router.message(
    (F.text.startswith('/user')) & 
    (F.chat.func(lambda chat: chat.type == 'private'))
)
async def user_base(
    message: Message,
    config: BotConfig,
) -> None:
    """User Base Info

    Args:
        message (Message): Tg Message
        user_class (User): Tortoise User Class
        admins (list): List of Game Admins
    """
    user_id = message.from_user.id
    # Check if the user is the owner
    if user_id not in config.admins:
        return
    splitted = message.text.split() 
    if len(splitted) == 1:
        users = await config.user_class.filter()
        text = f"{len(users)} registered users.\n"
        #names = [f"\n´{user.name.replace('_', '\_')}`" for user in users]
        for user in users:
            name = user.name#.replace("_", "\_")
            text += f"\n`{name}`"
    elif len(splitted) == 2:
        user = await config.user_class.get_or_none(name=splitted[1])
        if not user:
            text = f"No user found with {splitted[1]} name..."
        else:
            text = await user.show_data()
            text += f"\nUser ID: {user.tg_id}"
    await message.reply(text, parse_mode = "markdown")

@admin_router.message(
    (F.text.startswith('/random')) & 
    (F.chat.func(lambda chat: chat.type == 'private'))
)
async def random_user(
    message: Message, 
    config: BotConfig
) -> None: 
    if message.from_user.id not in config.admins:
        return
    
    users = await config.user_class.filter()

    selected = randint(0, len(users)-1)
    await message.reply(f"1 Random user was selected:\n@{users[selected].name}")



from setuptools import setup, find_packages

setup(
    name='funktgtools',
    version='0.1.0',
    description='Modular TG Features',
    url='https://github.com/funkaclau',
    author='funkaclau',
    packages=find_packages(),
    install_requires=[
        "aiogram==3.0.0b6"
    ],
)


from aiogram import types

async def send_poll(message: types.Message):
    args = message.text.split('\n')
    if len(args) < 3:
        await message.reply("You need to provide a poll question and at least two choices.")
        return

    question = args[0].replace('/poll ', '')
    choices = args[1:]

    await message.bot.send_poll(
        chat_id=message.chat.id, 
        question=question, 
        options=choices, 
        is_anonymous=False)
